gcc -I.. -DSLANG_DLL=1 -DWIN32 -L..\gw32objs sltest.c -lslang -luser32
sltest argv.sl
sltest syntax.sl
sltest scircuit.sl
sltest eqs.sl
sltest sscanf.sl
sltest loops.sl
sltest arith.sl
sltest array.sl
sltest strops.sl
sltest bstring.sl
sltest pack.sl
sltest stdio.sl
sltest assoc.sl
sltest selfload.sl
sltest struct.sl
sltest nspace.sl
sltest path.sl
sltest ifeval.sl
sltest anytype.sl
sltest arrmult.sl
sltest time.sl
sltest utf8.sl
sltest except.sl
sltest bugs.sl
sltest list.sl
sltest regexp.sl
sltest method.sl
sltest deref.sl
sltest naninf.sl
sltest overflow.sl
sltest sort.sl
sltest longlong.sl
sltest signal.sl
sltest dollar.sl
sltest req.sl
sltest docfun.sl
sltest debug.sl
sltest qualif.sl
sltest compare.sl
sltest break.sl
sltest multline.sl
sltest stack.sl
sltest misc.sl
sltest posixio.sl
sltest posdir.sl
sltest proc.sl
sltest math.sl
sltest autoload.sl
sltest nspace2.sl
sltest prep.sl
sltest argv.slc
sltest syntax.slc
sltest scircuit.slc
sltest eqs.slc
sltest sscanf.slc
sltest loops.slc
sltest arith.slc
sltest array.slc
sltest strops.slc
sltest bstring.slc
sltest pack.slc
sltest stdio.slc
sltest assoc.slc
sltest selfload.slc
sltest struct.slc
sltest nspace.slc
sltest path.slc
sltest ifeval.slc
sltest anytype.slc
sltest arrmult.slc
sltest time.slc
sltest utf8.slc
sltest except.slc
sltest bugs.slc
sltest list.slc
sltest regexp.slc
sltest method.slc
sltest deref.slc
sltest naninf.slc
sltest overflow.slc
sltest sort.slc
sltest longlong.slc
sltest signal.slc
sltest dollar.slc
sltest req.slc
sltest docfun.slc
sltest debug.slc
sltest qualif.slc
sltest compare.slc
sltest break.slc
sltest multline.slc
sltest stack.slc
sltest misc.slc
sltest posixio.slc
sltest posdir.slc
sltest proc.slc
sltest -utf8 math.slc
sltest -utf8 argv.sl
sltest -utf8 syntax.sl
sltest -utf8 scircuit.sl
sltest -utf8 eqs.sl
sltest -utf8 sscanf.sl
sltest -utf8 loops.sl
sltest -utf8 arith.sl
sltest -utf8 array.sl
sltest -utf8 strops.sl
sltest -utf8 bstring.sl
sltest -utf8 pack.sl
sltest -utf8 stdio.sl
sltest -utf8 assoc.sl
sltest -utf8 selfload.sl
sltest -utf8 struct.sl
sltest -utf8 nspace.sl
sltest -utf8 path.sl
sltest -utf8 ifeval.sl
sltest -utf8 anytype.sl
sltest -utf8 arrmult.sl
sltest -utf8 time.sl
sltest -utf8 utf8.sl
sltest -utf8 except.sl
sltest -utf8 bugs.sl
sltest -utf8 list.sl
sltest -utf8 regexp.sl
sltest -utf8 method.sl
sltest -utf8 deref.sl
sltest -utf8 naninf.sl
sltest -utf8 overflow.sl
sltest -utf8 sort.sl
sltest -utf8 longlong.sl
sltest -utf8 signal.sl
sltest -utf8 dollar.sl
sltest -utf8 req.sl
sltest -utf8 docfun.sl
sltest -utf8 debug.sl
sltest -utf8 qualif.sl
sltest -utf8 compare.sl
sltest -utf8 break.sl
sltest -utf8 multline.sl
sltest -utf8 stack.sl
sltest -utf8 misc.sl
sltest -utf8 posixio.sl
sltest -utf8 posdir.sl
sltest -utf8 proc.sl
sltest -utf8 math.sl
sltest -utf8 autoload.sl
sltest -utf8 nspace2.sl
sltest -utf8 prep.sl
sltest -utf8 argv.slc
sltest -utf8 syntax.slc
sltest -utf8 scircuit.slc
sltest -utf8 eqs.slc
sltest -utf8 sscanf.slc
sltest -utf8 loops.slc
sltest -utf8 arith.slc
sltest -utf8 array.slc
sltest -utf8 strops.slc
sltest -utf8 bstring.slc
sltest -utf8 pack.slc
sltest -utf8 stdio.slc
sltest -utf8 assoc.slc
sltest -utf8 selfload.slc
sltest -utf8 struct.slc
sltest -utf8 nspace.slc
sltest -utf8 path.slc
sltest -utf8 ifeval.slc
sltest -utf8 anytype.slc
sltest -utf8 arrmult.slc
sltest -utf8 time.slc
sltest -utf8 utf8.slc
sltest -utf8 except.slc
sltest -utf8 bugs.slc
sltest -utf8 list.slc
sltest -utf8 regexp.slc
sltest -utf8 method.slc
sltest -utf8 deref.slc
sltest -utf8 naninf.slc
sltest -utf8 overflow.slc
sltest -utf8 sort.slc
sltest -utf8 longlong.slc
sltest -utf8 signal.slc
sltest -utf8 dollar.slc
sltest -utf8 req.slc
sltest -utf8 docfun.slc
sltest -utf8 debug.slc
sltest -utf8 qualif.slc
sltest -utf8 compare.slc
sltest -utf8 break.slc
sltest -utf8 multline.slc
sltest -utf8 stack.slc
sltest -utf8 misc.slc
sltest -utf8 posixio.slc
sltest -utf8 posdir.slc
sltest -utf8 proc.slc
sltest -utf8 math.slc
